####################
# version 2.0
####################


################################################################################
#                                                                              #
# NAME Types.dict
# DESC Dict utilities
#                                                                              #
################################################################################
"""
:DOC:
Types.dict

:DESC:
Dict utilities

"""

# IMPORTS
#-------------------------------------------------------------------------------
from copy import copy
#-------------------------------------------------------------------------------
# IMPORTS

# CODE
#-------------------------------------------------------------------------------
def setdefaults(d, **kwargs):
    """
    d      -- dict
    kwargs -- key=value
    set all empty key to value
    """
    for key, value in kwargs.items():
        if not d.has_key(key):
            d[key] = value


def updated_copy(d1, d2):
    d = d1.copy()
    d.update(d2)
    return d


def select(d, selectFunction):
    """
    d              -- dict
    selectFunction -- function that take an item as argument and return True if item is selected or False

    return list of selected items
    """
    r = {}
    for k in d:
        if selectFunction(d[k]):
            r[k] = d[k]
    return r


def sum(d):
    return reduce(lambda x,y:x+y, d.values())


def avg(d):
    return reduce(lambda x,y:x+y, d.values()) / float(len(d.values()))


def sub(d, keys):
    r = {}
    for k in keys:
        r[k] = d[k]
    return r


def sorted_values(d):
    """Return values sorted by keys
    """
    keys = d.keys()
    keys.sort()
    return map(d.get, keys)


def sorted_keys(d):
    """Return list of keys sorted
    """
    keys = d.keys()
    keys.sort()
    return keys

def sorted_items_by_values(d):
    """Return list of tuple (key, value) sorted by values
    """
    l = d.items()
    l.sort( lambda x,y: x[1].__cmp__(y[1]))
    return l


def sorted_keys_values(d):
    """Return tuple of soterd_keys, sorted_values
    sorted by keys
    """
    keys = d.keys()
    keys.sort()
    return keys, map(d.get, keys)


def sorted_items(d):
    """Return list of tuple (key, items) sorted by keys
    """
    i = d.items()
    i.sort()
    return i


def trimed(d, cond=(lambda k,v:1)):
    """ Return a new dict in which cond(k,v) == true items are removed.
    d    -- dict to trim
    cond -- function with key and value args
    """   
    n={}
    for k,v in d.items():
        if not cond(k,v):
            n[k] = v
    return n


def add_values(d, d1):
    """
    return a dic t contening d and d1 keys where
    each key shared by d and d1 as d1 value + d value
    """
    r = copy(d)
    for k in d1.keys():
        if r.has_key(k):
            r[k] += d1[k]
        else:
            r[k] = d1[k]
    return r


def get_multi(d, keys, default=None):
    """
    get multiple values form dict d
    """
    return [d.get(k, default) for k in keys]
    

def merge_list(d1, d2):
    """
    d1 -- dict where values are list of items
    d2 -- dict where values are list of items    
    for all keys of d1,d2 extend d1 with values of d2 correponding to that key
   """
    keys = d1.keys()
    keys.extend(d2.keys())
    for k in keys:
        l = d1.get(k, [])
        l.extend( d2.get(k, []) )
    return d1





#-------------------------------------------------------------------------------
# CODE
