#
# python2.2 setup.py build_ext --inplace
#
cdef extern from "stdlib.h":
    ctypedef int size_t
    void* malloc(size_t size)
    void free(void* ptr)
    long int random()
    char* strcpy(char* to_str, char *from_str)

cdef extern from "math.h":
    cdef float log10(float)
    cdef float log(float)

import sys
cdef class Matrix:
    cdef float **m

    cdef int N
    cdef int M

    def __init__(self, matrix):

        self.N = int(matrix.ba)
        self.M = matrix.matrix.shape[0]


        # matrix malloc
        self.m = <float **> malloc(sizeof(float *) * self.M)
        
        for i in range(self.M):
            self.m[i] = <float *> malloc(sizeof(float) * 4)

        # copy matrix
        for a in range(matrix.matrix.shape[0]):
            for b in range(matrix.matrix.shape[1]):
                self.m[a][b] = matrix.matrix[a,b]


    def __dealloc__(self):
        for i in range(self.M):
            free(self.m[i])
        free(self.m)
        #print 'free matrix'



cdef class Sequence:
    cdef char *s
    cdef int L


    def __init__(self, sequence):
        self.L = len(sequence)

        # matrix malloc
        self.s =<char *> malloc(sizeof(char) * self.L)
    
cdef class Sequences:
    cdef char **s
    cdef int S


    cdef int *startLocation
    cdef int *endLocation

    def __init__(self, sequences):
        self.S = len(sequences)

        # matrix malloc
        self.s = <char **> malloc(sizeof(char *) * self.S)

        cdef int i,j

        #location
        self.startLocation = <int *> malloc(sizeof(int) * self.S)
        self.endLocation = <int *> malloc(sizeof(int) * self.S)

        cdef char *str
        cdef char c
        cdef int x
        for i from 0 <= i < self.S:
            self.s[i] = <char *> malloc(sizeof(char) * len(sequences[i].sequence))
            # cast python str into char *  (MUST USE SIMPLE VAR)

            str = tmp = sequences[i].sequence
            for j from 0 <= j < len(sequences[i].sequence):
                self.s[i][j] = str[j]


            self.startLocation[i] = x = v = sequences[i].location[0]
            self.endLocation[i] = x = v = sequences[i].location[1]



    def __dealloc__(self):
        free(self.startLocation)
        free(self.endLocation)

        cdef int i
        for i from 0 <= i < self.S:
            free(<char *> self.s[i])
        free(self.s)
        #print 'free sequences'


cdef class Bg:
    cdef int L

    cdef float *A
    cdef float *C
    cdef float *G
    cdef float *T
    
    cdef float *Threshold



    cdef int location[2]

    def __init__(self, bg):
        self.L = len(bg.threshold)
        self.Threshold = <float *>malloc(sizeof(float) * self.L)
        self.A = <float *>malloc(sizeof(float) * self.L)
        self.C = <float *>malloc(sizeof(float) * self.L)
        self.G = <float *>malloc(sizeof(float) * self.L)
        self.T = <float *>malloc(sizeof(float) * self.L)
        

        #for i in range(len(bg.threshold)):
        cdef int i
        for i from 0 <= i < self.L:
            self.Threshold[i] = bg.threshold[i] 
            self.A[i] = bg.pACGT[i,0]
            self.C[i] = bg.pACGT[i,1]
            self.G[i] = bg.pACGT[i,2]
            self.T[i] = bg.pACGT[i,3]
        self.location[0] = bg.location[0]
        self.location[1] = bg.location[1]


    def __dealloc__(self):
        free(self.Threshold)
        free(self.A)
        free(self.C)
        free(self.G)
        free(self.T)


def score_sequences_bg(Matrix cm, Sequences cs, Bg bg, overlap=False):

    cdef int L,M, N, S
    M = cm.M
    N = cm.N
    S = cs.S

    #score
    cdef float score, score_r

    # current letter
    cdef char X

    #result
    l = []

    cdef float pseudo
    #pseudo = max(1,cm.N / 10.0)

    cdef int i, j, k, I, J
    
    pseudo = cm.N * 0.01
    #print pseudo
    
    
    for k from 0 <= k < S:
        lk = []
        i = 0
        while i < (cs.endLocation[k] -  cs.startLocation[k] +1) -M+1: #for i in range(L-M+1):
            I = cs.startLocation[k] - bg.location[0] + i
            if bg.A[I] == 0.0 and bg.C[I] == 0.0:
                break
            # begin score matrix
            score = 0.0
            score_r = 0.0
            for j from 0 <= j < M:
                X = cs.s[k][i+j]
                if X == c'A':
                    score = score + log( (cm.m[j][0]*cm.N + bg.A[I]*pseudo) / (cm.N + pseudo) / bg.A[I] )
                    score_r = score_r + log( (cm.m[M-j-1][3]*cm.N + bg.T[I]*pseudo) / (cm.N + pseudo) / bg.T[I] )
                elif X == c'C':
                    score = score + log( (cm.m[j][1]*cm.N + bg.C[I]*pseudo) / (cm.N + pseudo) / bg.C[I] )
                    score_r = score_r + log( (cm.m[M-j-1][2]*cm.N + bg.G[I]*pseudo) / (cm.N + pseudo) / bg.G[I] )
                elif X == c'G':
                    score = score + log( (cm.m[j][2]*cm.N + bg.G[I]*pseudo) / (cm.N + pseudo) / bg.G[I] )
                    score_r = score_r + log( (cm.m[M-j-1][1]*cm.N + bg.C[I]*pseudo) / (cm.N + pseudo) / bg.C[I] )
                elif X == c'T':
                    score = score + log( (cm.m[j][3]*cm.N + bg.T[I]*pseudo) / (cm.N + pseudo) / bg.T[I] )
                    score_r = score_r + log( (cm.m[M-j-1][0]*cm.N + bg.A[I]*pseudo) / (cm.N + pseudo) / bg.A[I] )

            #end score matrix
            
            #print i, score, score_r, bg.Threshold[I]
            #print  i, bg.A[I], bg.C[I], bg.G[I], bg.T[I]
            #print score, bg.Threshold[I]
            #print i, score, score_r, bg.Threshold[I]
            
            if score >= bg.Threshold[I]:
                lk.append( (i, '+', score) )
                if not overlap:
                    i = i + M
                    continue
            if score_r >= bg.Threshold[I]:
                lk.append( (i, '-', score_r) )
                if not overlap:
                    i = i + M
                    continue

            i = i + 1
        
        l.append(lk)
    return l





