####################
# version 2.0
####################


"""
Score and hit distribution

methods:
    hits_per_seq_values
    hits_per_nseq_values
    hits_per_nseq_values_poisson
    score_values
    pvalues_values
    score_per_seq_values
    score_per_nseq_values
    
"""
import Core.types
import Stats.stats as stats

########################################
#                                      #
#             hit dist                 #
#                                      #
########################################
def hits_per_seq_values(h):
    """
    h -- list (sequences) of hits(=list of tuple)
    
    return values (value=number of hit per sequence)
    """
    return [len(i) for i in h]

def hits_per_nseq_values(h, n):
    """
    h -- list (sequences) of hits(=list of tuple)

    return values (value=number of hits per group of n sequences)
    """
    h = map(Core.types.list.flatten, Core.types.list.chunked(h, n)[:-1])
    return [len(i) for i in h]
        

def hits_per_nseq_values_poisson(h, n):
    """
    h -- list (sequences) of hits(=list of tuple)

    return values (value=number of hits per group of n sequences)
    """
    v = hits_per_nseq_values(h, n)
    return [int(i) for i in stats.r.rpois(len(v), stats.r.mean(v))]


########################################
#                                      #
#             score dist               #
#                                      #
########################################
def score_values(h):
    """
    h -- list (sequences) of hits(=list of tuple)
    
    return values (value=score of hit)
    """
    #return [i[2] for i in listUtils.flatList(h)]
    v = []
    for i in h:
        v += [j[2] for j in i]
    return v


def pvalues_values(h):
    """
    h -- list (sequences) of hits(=list of tuple)
    
    return values (value=-log(pvalue) of hit)
    """
    return [-i[3] for i in Core.types.list.flatten(h)]

    
def score_per_seq_values(h):
    """
    h -- list (sequences) of hits(=list of tuple)

    return values (value=cumulative score per sequence)
    """
    return [Core.types.list.sum([h[2] for h in i]) for i in h]
    
def score_per_nseq_values(h, n):
    """
    h -- list (sequences) of hits(=list of tuple)

    return values (value=cumulative score per group of n sequences)
    """
    h = map(Core.types.list.flatten, type.list.chunk(h, n)[:-1])
    return [Core.types.list.sum([i[2] for i in x]) for x in h]
    
def score_per_nhits_values(h, n):
    v = score_values(h)
    v = Core.types.list.chunked(v, n)[:-1]
    return map(Core.types.list.sum, v)

    
