####################
# version 2.0
####################


"""
Toolbox for scanning sequences

methods:
    scan
    build_S
    extract_positive_regions
    extract_growing_regions
    
"""

from math import *
from numpy import *

import Core.UI.cli as cli
import Core.Persistence.op as op




def scan(T_k, T_alpha, T_alpha_b, parameters):
    """
    ALG
    IN: k Tab, alpha Tab, alpha_b Tab
    OUT: list of tuples corrsponding to tab slices

    all tab have 2D:
    dim1: category or sequence
    dim2: position

    k : hit count tab
    alpha   : expected freq table for all sequences in group
    alpha_b : expected freq table in BG

    Note: all tab must have the same length = C number of categories

    parameters -- dict of parameters keys=(minSize, maxSize)
    """
    if T_alpha_b[0].all() == 0.0:
        cli.warning('Bad T_alpha table')
        return []

    t = build_S(T_k, T_alpha, T_alpha_b)
    return t

##
#
# Score function
# s_i(n,k_i, lambda, lambda_b)
#
##

def build_S(T_k, T_alpha, T_alpha_b):
    """
    build the score function
    """
    # length
    L = T_k.shape[1]
    
    S = array([0] * L, 'f')
    S[-1] = 0

    for i in range(len(S)):
        k = add.reduce( T_k[:,i] )
        alpha   = sum( T_alpha[:,i] )
        alpha_b = sum( T_alpha_b[:,i] )
        s_i = k * log( alpha / alpha_b ) + ( alpha_b - alpha )

        S[i] = max(0.0, S[i-1] + s_i)
        #print S[i-1], S[i]
    return S



########################################
#                                      #
#     extract regions from tab         #
#                                      #
########################################

def extract_positive_regions(tab, minSize=50):
    """
    return list of tuple [(i,j), (i,j)] corresponding to regions where score is positive

    tab -- the score function
    """
    l = []
    a = -1
    for i in range(len(tab)):
        if tab[i] > 0:
            if a == -1:
                a = i
            b = i
        if tab[i] <= 0 or i == len(tab)-1:
            if a != -1 and (b-a+1) >= minSize:
                l += [ (a,b) ]
            a = -1

    return l

def extract_growing_regions(tab, minSize=20, maxSize=1000):
    """Return list of tuple [(i,j), (i,j)] corresponding to regions with posititve score
    Note: right bound is excluded: [i,j[

    tab -- the score function
    """

    l = []
    a = -1
    b = -1
    maximum = 0.0
    for i in range(len(tab)):
        if tab[i] > 0.0:
            if a == -1:
                a = i
            #b = i
            if tab[i] >= maximum:
                maximum = tab[i]
                b       = i
        if tab[i] <= 0.0 or i >= len(tab)-1:
            if a != -1 and (b-a+1) >= minSize and (b-a+1) <= maxSize:
                l += [ (a,b+1) ]
                maximum = 0.0
            a = -1
    return l

